@@ -14,7 +14,7 @@
 ##############################################################################
 """ Unit tests for GenericSetup-based export / import of PluginRegistry.
 
-$Id: test_exportimport.py 68733 2006-06-18 11:44:00Z jens $
+$Id: test_exportimport.py 74716 2007-04-24 19:15:04Z tseaver $
 """
 import unittest
 
@@ -33,6 +33,17 @@ else:
     from Products.GenericSetup.utils import _getDottedName
 
     from zope.interface import Interface
+    from zope.interface import directlyProvides
+    from zope.app.testing import ztapi
+
+    try:
+        from zope.traversing.interfaces import ITraversable
+        from zope.traversing.interfaces import TraversalError
+    except ImportError:
+        # BBB for Zope 2.9
+        from zope.app.traversing.interfaces import ITraversable
+        from zope.app.traversing.interfaces import TraversalError
+
     try:
         from zope.app.testing.placelesssetup import PlacelessSetup
     except ImportError:
@@ -120,6 +131,9 @@ else:
             registry._plugins = {} # it is usually lazy
 
             for plugin_type, registered in plugins.items():
+                for obj_id in registered:
+                    obj = app._getOb(obj_id)
+                    directlyProvides(obj, plugin_type)
                 registry._plugins[plugin_type] = registered
 
             app._setObject('plugin_registry', registry)
@@ -134,6 +148,7 @@ else:
             return PluginRegistryExporter
 
         def test_empty(self):
+            ztapi.provideAdapter(None, ITraversable, AttrItemTraverser)
 
             app, registry = self._initRegistry()
             exporter = self._makeOne(registry).__of__(registry)
@@ -142,6 +157,8 @@ else:
             self._compareDOM(xml, _EMPTY_PLUGINREGISTRY_EXPORT)
 
         def test_normal_no_plugins(self):
+            ztapi.provideAdapter(None, ITraversable, AttrItemTraverser)
+
             app, registry = self._initRegistry(
                                     plugin_type_info=_PLUGIN_TYPE_INFO)
             exporter = self._makeOne(registry).__of__(registry)
@@ -150,6 +167,8 @@ else:
             self._compareDOM(xml, _NO_PLUGINS_PLUGINREGISTRY_EXPORT)
 
         def test_normal_with_plugins(self):
+            ztapi.provideAdapter(None, ITraversable, AttrItemTraverser)
+
             app, registry = self._initRegistry(
                                     plugin_type_info=_PLUGIN_TYPE_INFO,
                                     plugins={IFoo: ('foo_plugin_1',
@@ -165,6 +184,7 @@ else:
         def test_empty(self):
             from Products.PluginRegistry.exportimport \
                 import exportPluginRegistry
+            ztapi.provideAdapter(None, ITraversable, AttrItemTraverser)
 
             app, registry = self._initRegistry()
             context = DummyExportContext(app)
@@ -179,6 +199,7 @@ else:
         def test_normal_no_plugins(self):
             from Products.PluginRegistry.exportimport \
                 import exportPluginRegistry
+            ztapi.provideAdapter(None, ITraversable, AttrItemTraverser)
 
             app, registry = self._initRegistry(
                                     plugin_type_info=_PLUGIN_TYPE_INFO)
@@ -194,6 +215,7 @@ else:
         def test_normal_with_plugins(self):
             from Products.PluginRegistry.exportimport \
                 import exportPluginRegistry
+            ztapi.provideAdapter(None, ITraversable, AttrItemTraverser)
 
             app, registry = self._initRegistry(
                                     plugin_type_info=_PLUGIN_TYPE_INFO,
@@ -346,7 +368,9 @@ else:
             from Products.PluginRegistry.exportimport \
                 import importPluginRegistry
 
-            app, registry = self._initRegistry()
+            app, registry = self._initRegistry(plugins={IFoo: ('foo_plugin_1',
+                                                               'foo_plugin_2')},
+                                              )
 
             self.assertEqual(len(registry.listPluginTypeInfo()), 0)
             self.assertRaises(KeyError, registry.listPlugins, IFoo)
@@ -377,7 +401,9 @@ else:
             from Products.PluginRegistry.exportimport \
                 import importPluginRegistry
 
-            app, registry = self._initRegistry()
+            app, registry = self._initRegistry(plugins={IFoo: ('foo_plugin_1',
+                                                               'foo_plugin_2')},
+                                              )
 
             self.assertEqual(len(registry.listPluginTypeInfo()), 0)
             self.assertRaises(KeyError, registry.listPlugins, IFoo)
@@ -410,6 +436,43 @@ else:
 
             self.assertEqual(len(registry.listPlugins(IBar)), 0)
 
+        def test_normal_with_plugins_skip_duplicates(self):
+            # See http://www.zope.org/Collectors/PAS/52
+            from Products.PluginRegistry.exportimport \
+                import importPluginRegistry
+
+            app, registry = self._initRegistry()
+
+            self.assertEqual(len(registry.listPluginTypeInfo()), 0)
+            self.assertRaises(KeyError, registry.listPlugins, IFoo)
+            self.assertRaises(KeyError, registry.listPlugins, IBar)
+
+            context = DummyImportContext(app, False)
+            context._files['pluginregistry.xml'
+                          ] = _NORMAL_PLUGINREGISTRY_EXPORT
+
+            importPluginRegistry(context)
+            importPluginRegistry(context) # twice should not duplicate
+
+            self.assertEqual(len(registry.listPluginTypeInfo()), 2)
+
+    class AttrItemTraverser:
+        _marker = object()
+
+        def __init__(self, context):
+            self.context = context
+
+        def traverse(self, name, furtherPath):
+            result = getattr(self.context, name, self._marker)
+            if result is self._marker:
+                try:
+                    result = self.context.get(name, self._marker)
+                except AttributeError:
+                    pass
+            if result is self._marker:
+                raise TraversalError(name)
+            return result
+
     def test_suite():
         return unittest.TestSuite((
             unittest.makeSuite( PluginRegistryExporterTests ),
